@@ -6,7 +6,7 @@ structable <- function(x, ...)
 
 structable.formula <- function(formula, data = NULL, direction = NULL,
                                split_vertical = NULL, ..., subset, na.action) {
-    if (missing(formula) || !inherits(formula, "formula")) 
+    if (missing(formula) || !inherits(formula, "formula"))
         stop("formula is incorrect or missing")
 
     m <- match.call(expand.dots = FALSE)
@@ -19,7 +19,7 @@ structable.formula <- function(formula, data = NULL, direction = NULL,
       split_vertical <- attr(data, "split_vertical")
       data <- as.table(data)
     }
-    
+
     if (is.null(split_vertical))
       split_vertical <- FALSE
 
@@ -28,15 +28,15 @@ structable.formula <- function(formula, data = NULL, direction = NULL,
     ## only rhs present without `.' in lhs => xtabs-interface
     if (length(formula) != 3) {
       if (formula[[1]] == "~") {
-        if (inherits(edata, "ftable") || inherits(edata, "table") || 
+        if (inherits(edata, "ftable") || inherits(edata, "table") ||
             length(dim(edata)) > 2) {
           data <- as.table(data)
           varnames <- attr(terms(formula, allowDotAsName = TRUE), "term.labels")
           dnames <- names(dimnames(data))
           di <- match(varnames, dnames)
-          if (any(is.na(di))) 
+          if (any(is.na(di)))
             stop("incorrect variable names in formula")
-          if (all(varnames != ".")) 
+          if (all(varnames != "."))
             data <- margin.table(data, di)
           return(structable(data, split_vertical = split_vertical, ...))
         }
@@ -47,9 +47,9 @@ structable.formula <- function(formula, data = NULL, direction = NULL,
                               split_vertical = split_vertical, ...))
           else
             return(structable(xtabs(formula, data),  split_vertical = split_vertical, ...))
-            
+
         } else {
-          if (is.matrix(edata)) 
+          if (is.matrix(edata))
             m$data <- as.data.frame(data)
           m$... <- m$split_vertical <- m$direction <- NULL
           m[[1]] <- as.name("model.frame")
@@ -62,45 +62,45 @@ structable.formula <- function(formula, data = NULL, direction = NULL,
     }
 
     ## `ftable' behavior
-    if (any(attr(terms(formula, allowDotAsName = TRUE), "order") > 1)) 
+    if (any(attr(terms(formula, allowDotAsName = TRUE), "order") > 1))
         stop("interactions are not allowed")
     rvars <- attr(terms(formula[-2], allowDotAsName = TRUE), "term.labels")
     cvars <- attr(terms(formula[-3], allowDotAsName = TRUE), "term.labels")
     rhs.has.dot <- any(rvars == ".")
     lhs.has.dot <- any(cvars == ".")
-    if (lhs.has.dot && rhs.has.dot) 
+    if (lhs.has.dot && rhs.has.dot)
         stop(paste("formula has", sQuote("."), "in both left and right hand side"))
-    if (inherits(edata, "ftable") || inherits(edata, "table") || 
+    if (inherits(edata, "ftable") || inherits(edata, "table") ||
         length(dim(edata)) > 2) {
         if (inherits(edata, "ftable"))
             data <- as.table(data)
-        
+
         dnames <- names(dimnames(data))
         rvars <- pmatch(rvars, dnames)
         cvars <- pmatch(cvars, dnames)
-        if (rhs.has.dot) 
+        if (rhs.has.dot)
             rvars <- seq_along(dnames)[-cvars]
-        else if (any(is.na(rvars))) 
+        else if (any(is.na(rvars)))
           stop("incorrect variable names in rhs of formula")
-        if (lhs.has.dot) 
+        if (lhs.has.dot)
             cvars <- seq_along(dnames)[-rvars]
-        else if (any(is.na(cvars))) 
+        else if (any(is.na(cvars)))
           stop("incorrect variable names in lhs of formula")
         split_vertical <- c(rep(FALSE, length(rvars)), rep(TRUE, length(cvars)))
         structable(margin.table(data, c(rvars, cvars)), split_vertical = split_vertical, ...)
     } else {
-        if (is.matrix(edata)) 
+        if (is.matrix(edata))
             m$data <- as.data.frame(data)
         m$... <- m$split_vertical <- m$direction <- NULL
         if (!is.null(data) && is.environment(data)) {
             dnames <- names(data)
-            if (rhs.has.dot) 
+            if (rhs.has.dot)
                 rvars <- seq_along(dnames)[-cvars]
-            if (lhs.has.dot) 
+            if (lhs.has.dot)
                 cvars <- seq_along(dnames)[-rvars]
         }
         else {
-            if (lhs.has.dot || rhs.has.dot) 
+            if (lhs.has.dot || rhs.has.dot)
                 stop("cannot use dots in formula with given data")
         }
         if ("Freq" %in% colnames(m$data))
@@ -113,19 +113,19 @@ structable.formula <- function(formula, data = NULL, direction = NULL,
         structable(mf, split_vertical = split_vertical, ...)
     }
 }
-  
+
 
 structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   ## several checks & transformations for arguments
   args <- list(...)
-  
-  if (length(args) == 0) 
+
+  if (length(args) == 0)
     stop("Nothing to tabulate")
-  
+
   x <- args[[1]]
-  x <- if (is.list(x)) 
+  x <- if (is.list(x))
     table(x)
-  else if (inherits(x, "ftable")) 
+  else if (inherits(x, "ftable"))
     as.table(x)
   else if (!(is.array(x) && length(dim(x)) > 1 || inherits(x, "table")))
     do.call("table", as.list(substitute(list(...)))[-1])
@@ -140,7 +140,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
     split_vertical <- rep(c(split_vertical, !split_vertical), length.out = dl)
   if (length(split_vertical) < dl)
     split_vertical <- rep(split_vertical, length.out = dl)
-    
+
 
   ## permute & reshape
   ret <- aperm(x, c(rev(which(!split_vertical)), rev(which(split_vertical))))
@@ -151,7 +151,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   rl <- if (length(rv)) sapply(rv, length) else 1
   cl <- if (length(cv)) sapply(cv, length) else 1
   dim(ret) <- c(prod(cl), prod(rl))
-  
+
   ## add dimnames
   attr(ret, "dnames") <- dn
   attr(ret, "split_vertical") <- split_vertical
@@ -168,7 +168,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   args <- if (nargs() < 3)
     list(..1)
   else
-    list(..1, ..2)
+    .massage_args(...)
 
   args <- lapply(args, function(x) if (is.logical(x)) which(x) else x)
 
@@ -178,17 +178,17 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
       ## resolve calls like x[[c(1,2)]]
       return(x[[ args[[1]][1] ]] [[ args[[1]][-1] ]])
     else
-      ## resolve x[[foo]] 
+      ## resolve x[[foo]]
       return(if (attr(x, "split_vertical")[1]) x[[,args[[1]] ]] else x[[args[[1]],]])
 
   ## handle calls like x[[c(1,2), c(3,4)]]
   if (length(args[[1]]) > 1 && length(args[[2]]) > 1)
     return(x[[ args[[1]][1], args[[2]][1] ]] [[ args[[1]][-1], args[[2]][-1] ]])
-  
+
   ## handle calls like x[[c(1,2), 3]]
   if (length(args[[1]]) > 1)
     return(x[[ args[[1]][1], args[[2]] ]] [[ args[[1]][-1], ]])
-  
+
   ## handle calls like x[[1, c(1,3)]]
   if (length(args[[2]]) > 1)
     return(x[[ args[[1]], args[[2]][1] ]] [[ , args[[2]][-1] ]])
@@ -227,7 +227,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
     split <- split[-i]
     dnames <- dnames[-i]
   }
-    
+
   if (!rsym) {
     i <- which(split)[1]
     split <- split[-i]
@@ -236,7 +236,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
 
   attr(ret, "split_vertical") <- split
   attr(ret, "dnames") <- dnames
-  
+
   ## add dimension attributes in ftable-format
   attr(ret, "col.vars") <- dnames[split]
   attr(ret, "row.vars") <- dnames[!split]
@@ -249,7 +249,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   args <- if (nargs() < 4)
     list(..1)
   else
-    list(..1, ..2)
+    .massage_args(...)
 
   ## handle one-arg cases
   if (nargs() < 4)
@@ -258,25 +258,25 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
                Recall(x, args[[1]][1],
                       value = Recall(x[[ args[[1]][1] ]], args[[1]][-1], value = value))
            else
-               ## resolve x[[foo]]<-value 
+               ## resolve x[[foo]]<-value
                if (attr(x, "split_vertical")[1])
                    Recall(x,,args[[1]], value = value)
                else
                    Recall(x,args[[1]],, value = value)
            )
-  
+
   ## handle calls like x[[c(1,2), c(3,4)]]<-value
   if (length(args[[1]]) > 1 && length(args[[2]]) > 1)
     return(Recall(x, args[[1]][1], args[[2]][1],
                   value = Recall(x[[ args[[1]][1], args[[2]][1] ]],
                     args[[1]][-1], args[[2]][-1], value = value)))
-  
+
   ## handle calls like x[[c(1,2), 3]]<-value
   if (length(args[[1]]) > 1)
     return(Recall(x, args[[1]][1], args[[2]],
                   value = Recall(x[[ args[[1]][1], args[[2]] ]],
                     args[[1]][-1], ,value = value)))
-  
+
   ## handle calls like x[[1, c(1,3)]]<-value
   if (length(args[[2]]) > 1)
     return(Recall(x, args[[1]], args[[2]][1],
@@ -322,7 +322,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   args <- if (nargs() < 3)
     list(..1)
   else
-    list(..1, ..2)
+    .massage_args(...)
 
   args <- lapply(args, function(x) if (is.logical(x)) which(x) else x)
 
@@ -333,7 +333,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   ## handle calls like x[c(1,2), foo]
   if (length(args[[1]]) > 1)
     return(do.call(rbind, lapply(args[[1]], function(i) x[i, args[[2]]])))
-  
+
   ## handle calls like x[foo, c(1,3)]
   if (length(args[[2]]) > 1)
     return(do.call(cbind, lapply(args[[2]], function(i) x[args[[1]], i])))
@@ -371,7 +371,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
     i <- which(!split)[1]
     dnames[[i]] <- dnames[[i]][args[[1]]]
   }
-    
+
   if (!rsym) {
     i <- which(split)[1]
     dnames[[i]] <- dnames[[i]][args[[2]]]
@@ -379,7 +379,7 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
 
   attr(ret, "split_vertical") <- split
   attr(ret, "dnames") <- dnames
-  
+
   ## add dimension attributes in ftable-format
   attr(ret, "col.vars") <- dnames[split]
   attr(ret, "row.vars") <- dnames[!split]
@@ -392,31 +392,31 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   args <- if (nargs() < 4)
     list(..1)
   else
-    list(..1, ..2)
+    .massage_args(...)
 
   ## handle one-arg cases
   if (nargs() < 4)
-    return(## resolve x[foo] 
+    return(## resolve x[foo]
            if (attr(x, "split_vertical")[1])
              Recall(x,,args[[1]], value = value)
            else
              Recall(x,args[[1]],, value = value)
            )
-  
+
   ## handle calls like x[c(1,2), 3]
   if (length(args[[1]]) > 1) {
     for (i in seq_along(args[[1]]))
       x[ args[[1]][i], args[[2]] ] <- value[i,]
     return(x)
   }
-    
+
   ## handle calls like x[1, c(2,3)]
   if (length(args[[2]]) > 1) {
     for (i in seq_along(args[[2]]))
       x[ args[[1]], args[[2]][i] ] <- value[,i]
     return(x)
   }
-    
+
   ## final cases like x[1,2] or x[1,] or x[,1]
   dnames <- attr(x, "dnames")
   split <- attr(x, "split_vertical")
@@ -502,7 +502,7 @@ rbind.structable <- function(..., deparse.level = 1) {
 }
 
 as.table.structable <- function(x, ...) {
-  ret <- stats:::as.table.ftable(x) 
+  ret <- stats:::as.table.ftable(x)
   structure(aperm(ret, match(names(dimnames(ret)), names(attr(x, "dnames")))),
             class = "table")
 }
@@ -534,13 +534,20 @@ dimnames.structable <- function(x) attr(x,"dnames")
 
 as.vector.structable <- function(x, ...)
   as.vector(as.table(x), ...)
-  
-as.matrix.structable <- function(x, ...) 
+
+as.matrix.structable <- function(x, ...)
   matrix(as.vector(unclass(x)), ncol = attr(x, "dim")[2])
 
 length.structable <- function(x) dim(x)[1]
 
-is.na.structable <- function(x) 
+is.na.structable <- function(x)
   sapply(seq_along(x), function(sub) any(is.na(sub)))
 
 
+############# helper function
+.massage_args <- function(...) {
+    args <- vector("list", 2)
+    args[[1]] <- if(missing(..1)) as.symbol("grrr") else ..1
+    args[[2]] <- if(missing(..2)) as.symbol("grrr") else ..2
+    args
+}
